
# q-expansion of newform 1386.2.k.w, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1386
weight = 2
poly_data = [4, -18, 27, -19, 12, -3, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[3, -1, 1, 0, 0, 0], 1], [[-18, 43, -28, 22, -5, 2], 2], [[22, -56, 37, -31, 7, -3], 1], [[50, -121, 82, -64, 15, -6], 2], [[24, -62, 44, -33, 8, -3], 1]]

hecke_ring_character_values = [[155, [1, 0, 0, 0, 0, 0]], [199, [0, 0, 1, 0, 0, 0]], [1135, [1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, -1, 0, 0, 0], [0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 1, 0], [0, 0, 0, 1, -1, 1], [1, 0, 1, 0, 0, 0], [0, 2, 0, 0, 0, 0], [1, 0, 1, 0, 0, 0], [1, 1, 0, 1, 0, 2], [-1, -1, -4, -1, 1, -2], [-4, 0, 0, 1, 0, -1], [-2, 2, 0, 4, -4, 2], [1, 1, 4, 1, 0, 2], [-5, 0, 0, 1, 0, -1], [2, -2, 0, 1, 0, -1], [-1, -1, 4, -1, 1, -2], [0, 2, 0, 0, -2, 0], [3, -3, 4, 2, 2, 1], [-2, -2, -4, -2, 5, -4], [-2, 1, -1, 2, -2, 1], [-2, 0, 0, 1, 0, -1], [0, 0, -2, -4, 2, -2], [0, 0, 4, 0, -3, 0], [3, 4, 0, -1, 0, 1], [0, 0, 12, 0, 2, 0], [1, 0, 0, 2, 0, -2], [-5, 5, -6, -2, -4, -1], [2, 2, -2, 2, 0, 4], [-1, -1, -9, -1, 6, -2], [-2, -3, -4, -4, 5, -2], [6, 0, 0, -2, 0, 2], [2, -3, 0, 1, 0, -1], [2, 2, 8, 2, -8, 4], [0, 2, 0, 0, -2, 0], [12, 0, 0, -3, 0, 3], [1, 1, -8, 1, 0, 2], [-1, 4, 0, 2, -5, 1], [3, -3, 6, 6, 0, 3], [-1, -1, -5, -1, 6, -2], [-6, -6, 0, 2, 0, -2], [2, 2, 16, 2, -4, 4], [-9, -3, -10, -2, 4, -1], [12, -2, 0, -2, 0, 2], [0, 0, -8, 0, 4, 0], [2, 4, 0, -4, -2, -2], [-4, 2, 0, -3, 0, 3], [12, 2, 10, -4, 0, -2], [8, 2, 0, -4, 0, 4], [-2, 2, 0, 2, 0, -2], [4, 1, 5, 2, -2, 1], [2, 2, 0, 2, -2, 4], [-2, -2, -9, -2, 0, -4], [0, -2, 0, 4, 0, -4], [0, 6, -2, -4, -4, -2], [8, 6, 0, -1, 0, 1], [0, 0, 16, 0, 2, 0], [-10, 6, -14, -8, -2, -4], [-10, -5, -12, -4, 7, -2], [0, 0, 8, 0, -4, 0], [-8, -6, -8, 0, 6, 0], [11, 0, 0, -2, 0, 2], [-4, 6, 0, 8, -10, 4], [10, 0, 0, -3, 0, 3], [-2, -4, 0, 2, 0, -2], [-3, 4, -4, -2, -3, -1], [1, 1, 26, 1, -2, 2], [4, 4, 8, 4, -9, 8], [-5, -5, -5, -5, 10, -10], [0, 10, 0, -2, 0, 2], [-14, -5, -15, -2, 6, -1], [12, 5, 0, 0, 0, 0], [-16, -4, -18, -4, 6, -2], [0, 0, 10, 0, -8, 0], [-20, -6, -20, 0, 6, 0], [2, 2, 4, 2, 3, 4], [-1, 4, 0, 1, 0, -1], [5, 5, 0, 5, -6, 10], [-4, 1, -4, 0, -1, 0], [-3, -3, 10, -3, 2, -6], [-2, -2, 0, -2, 6, -4], [18, -4, 12, -12, 10, -6], [0, 6, 0, 1, 0, -1], [-20, -2, 0, 3, 0, -3], [4, -10, 8, 8, 6, 4], [11, -4, 0, -2, 0, 2], [-3, -3, -4, -3, -1, -6], [-1, -1, 18, -1, 6, -2], [-4, 2, 0, 4, 0, -4], [-4, -4, -14, -4, 0, -8], [28, 4, 0, -5, 0, 5], [-22, -2, 0, 2, 0, -2], [-3, -3, -24, -3, 2, -6], [-2, 6, -2, 0, -6, 0], [28, -4, 30, 4, 2, 2], [3, 12, 0, -5, 0, 5], [-2, -2, 0, -2, -8, -4], [-4, -6, 0, 2, 0, -2], [-4, -4, 0, -4, 2, -8], [-2, -8, -2, 0, 8, 0], [-4, -4, 16, -4, 10, -8], [-4, -4, 12, -4, 3, -8], [2, -2, 0, 3, 0, -3], [-23, -1, -26, -6, 4, -3], [0, 0, 0, 0, 0, 0], [-3, -3, -18, -3, 10, -6], [-9, -1, -10, -2, 2, -1], [-12, 7, -9, 6, -10, 3], [30, 2, 0, 2, 0, -2], [-5, -5, 6, -5, 10, -10], [-4, -1, -8, -8, 5, -4], [24, 0, 0, -2, 0, 2], [-4, -4, 8, -4, 3, -8], [0, 5, 4, 8, -9, 4], [-4, -2, 0, 6, 0, -6], [-16, 9, -19, -6, -6, -3], [2, 2, 0, 6, 0, -6], [-10, 2, -12, -4, 0, -2], [-16, 0, 0, 6, 0, -6], [12, -5, 16, 8, 1, 4], [-4, -4, -4, -4, 7, -8], [-35, -4, 0, -1, 0, 1], [11, 5, 10, -2, -4, -1], [-2, -4, 0, 6, 0, -6], [3, 3, 28, 3, 0, 6], [-5, -7, -12, -14, 14, -7], [-3, -3, -3, -3, 10, -6], [-2, -2, 0, -3, 0, 3], [-1, -1, -16, -1, 2, -2], [-1, -1, 8, -1, -3, -2], [2, -8, 0, 4, 0, -4], [2, 2, 8, 2, -1, 4], [16, -6, 10, -12, 12, -6], [32, -2, 0, 2, 0, -2], [2, 2, -8, 2, 6, 4], [4, 4, 0, 3, 0, -3], [-1, -1, -21, -1, 2, -2], [28, 6, 0, -10, 0, 10], [-26, -9, -28, -4, 11, -2], [-7, 1, -16, -18, 8, -9], [0, 7, 0, 2, 0, -2], [-16, 1, -15, 2, -2, 1], [-42, 0, 0, 2, 0, -2], [2, 2, -8, 2, -8, 4], [6, 0, 12, 12, -6, 6], [-21, 0, 0, 5, 0, -5], [-47, 1, -48, -2, 0, -1], [0, -9, 0, 8, 0, -8], [12, 3, 0, 0, 0, 0], [5, 0, 9, 8, -4, 4], [1, 1, -1, 1, 2, 2], [4, 4, 32, 4, -5, 8], [-4, -4, -4, -4, 11, -8], [2, 0, 0, -2, 0, 2], [17, 0, 0, -9, 0, 9], [6, 6, -6, 6, -6, 12], [6, -9, 9, 6, 6, 3], [-14, 15, 0, -1, 0, 1], [-5, -5, -24, -5, -1, -10], [6, 6, -4, 6, -14, 12], [-14, 0, 0, 6, 0, -6], [-33, 3, -30, 6, -6, 3], [-5, -5, 6, -5, 14, -10], [-15, 12, 0, -5, 0, 5], [30, -5, 0, 1, 0, -1], [6, 6, 18, 6, -8, 12], [-12, 4, -4, 16, -12, 8], [-33, 4, -36, -6, -1, -3], [-8, -10, -8, 0, 10, 0], [-40, -2, -40, 0, 2, 0], [20, 6, 0, 0, 0, 0], [-6, -6, -8, -6, 10, -12], [-8, 7, -13, -10, -2, -5], [4, 18, 0, 2, 0, -2], [12, 12, 10, 12, -16, 24], [-20, 10, -28, -16, -2, -8], [28, -12, 34, 12, 6, 6], [0, -18, 0, 2, 0, -2], [12, 8, 0, -1, 0, 1], [26, 8, 28, 4, -10, 2], [-20, 5, 0, 8, 0, -8], [-4, -4, -20, -4, 14, -8], [-4, -4, -12, -4, -2, -8], [-9, -12, 0, 1, 0, -1], [10, 10, 0, 5, 0, -5], [2, 2, 1, 2, -8, 4], [7, 5, 16, 18, -14, 9], [-20, 10, -20, 0, -10, 0], [-4, 9, 0, 8, -13, 4], [22, -3, 23, 2, 2, 1], [4, 4, 40, 4, -8, 8], [-10, 8, -12, -4, -6, -2], [5, 5, 34, 5, -6, 10], [7, -12, 0, -3, 0, 3], [-1, -1, -11, -1, -6, -2], [-2, -2, 52, -2, 2, -4], [18, 2, 20, 4, -4, 2], [-38, 0, -38, 0, 0, 0], [-40, -6, -38, 4, 4, 2], [3, 3, 20, 3, -6, 6], [-2, 16, 0, 0, 0, 0], [-6, -6, -44, -6, 4, -12], [2, -12, -6, -16, 20, -8], [-6, 12, 0, -5, 0, 5], [6, 6, 20, 6, -2, 12], [16, 8, 20, 8, -12, 4], [12, -4, 0, -11, 0, 11], [-12, 10, -12, 0, -10, 0], [8, 8, -4, 8, -12, 16], [5, 5, 9, 5, -22, 10], [-13, -16, 0, -3, 0, 3], [-42, 2, -40, 4, -4, 2], [0, 0, 22, 0, -8, 0], [-12, 9, 0, -8, 0, 8], [-18, -2, 0, -2, 0, 2], [-6, -6, -14, -6, 6, -12], [32, 2, 28, -8, 2, -4], [1, 1, 22, 1, -14, 2], [8, 4, 4, -8, 0, -4], [12, 15, 11, -2, -14, -1], [-4, -4, 20, -4, 5, -8], [-16, -10, 0, 10, 0, -10], [6, 6, 28, 6, -14, 12], [-6, -3, 0, -5, 0, 5], [6, 6, -8, 6, 6, 12], [-2, -2, 36, -2, 0, -4], [-19, 4, 0, 5, 0, -5], [20, 1, 0, -8, 0, 8], [-4, -4, -22, -4, 2, -8], [-5, -8, -16, -22, 19, -11], [7, 7, 28, 7, -1, 14], [-4, -4, -40, -4, 4, -8], [-23, 3, -20, 6, -6, 3], [-15, 7, -22, -14, 0, -7], [18, -4, 0, -1, 0, 1], [25, -2, 23, -4, 4, -2], [-3, 4, 0, -5, 0, 5], [-8, -4, -2, 12, -2, 6], [-3, -3, 2, -3, -18, -6], [-4, -4, -10, -4, 14, -8], [8, 2, 0, -4, 0, 4], [-16, 5, 0, -4, 0, 4], [19, 13, 24, 10, -18, 5], [8, 8, 0, 8, -16, 16], [50, -2, 48, -4, 4, -2], [1, 1, 6, 1, -2, 2], [-44, 12, 0, -5, 0, 5], [-4, -4, -4, -4, 2, -8], [-10, 28, 0, -10, 0, 10], [9, 9, 6, -6, -6, -3], [21, -13, 14, -14, 20, -7], [-36, -12, 0, 4, 0, -4], [-4, -6, 0, 0, 0, 0], [3, 3, 26, 3, -10, 6], [31, -6, 40, 18, -3, 9], [11, -12, 0, 9, 0, -9], [-14, 16, -14, 0, -16, 0], [-6, -6, -4, -6, 4, -12], [2, -18, -2, -8, 22, -4], [-31, -11, -38, -14, 18, -7], [-8, -2, 0, 6, 0, -6], [-6, -6, 18, -6, 10, -12], [4, 17, 12, 16, -25, 8], [-4, 4, 0, 6, 0, -6], [12, 3, 16, 8, -7, 4], [32, -3, 0, 2, 0, -2], [-14, 10, -18, -8, -6, -4], [-2, -2, -12, -2, -12, -4], [24, 9, 0, -14, 0, 14], [-16, -4, 0, -2, 0, 2], [-50, 6, 0, 1, 0, -1], [22, 2, 30, 16, -10, 8], [-11, -11, -22, -11, 16, -22], [30, -1, 25, -10, 6, -5], [-38, -8, -38, 0, 8, 0], [-3, -3, 20, -3, 3, -6], [14, 2, 0, 8, 0, -8], [-2, -2, 36, -2, -6, -4], [9, 9, 20, 9, -8, 18], [-26, -9, -28, -4, 11, -2], [-4, -4, -34, -4, -6, -8], [-7, -7, 13, -7, 6, -14], [10, -4, 22, 24, -8, 12], [12, 18, 14, 4, -20, 2], [8, 8, 0, -6, 0, 6], [-24, 13, 0, 2, 0, -2], [-5, -5, -26, -5, -4, -10], [9, 9, -24, 9, -9, 18], [2, -4, 6, 8, 0, 4], [-48, -2, 0, 5, 0, -5], [-4, 2, 6, 20, -12, 10], [8, -16, 0, -1, 0, 1], [6, -9, 16, 20, -1, 10], [21, 25, 24, 6, -28, 3], [-5, -5, -13, -5, 18, -10], [-31, -8, 0, 7, 0, -7], [8, -19, 0, 2, 0, -2], [42, -12, 44, 4, 10, 2], [6, 6, -34, 6, -4, 12], [-4, -13, 0, 10, 0, -10], [-5, -5, -5, -5, 6, -10], [8, 0, 0, -13, 0, 13], [-8, -8, -26, -8, 26, -16], [3, 3, -26, 3, 14, 6], [9, 3, 8, -2, -2, -1]]
